@@ -13,7 +13,7 @@ use Carp;
 require Tk::ListBrowser::SelectXPM;
 use Math::Round qw(round);
 
-$VERSION = 0.04;
+$VERSION = 0.08;
 
 use base qw(Tk::ListBrowser::BaseItem);
 
@@ -30,8 +30,9 @@ my $dlmreg = qr/\.|\(|\)|\:|\!|\+|\,|\-|\<|\=|\>|\%|\&|\*|\"|\'|\/|\;|\?|\[|\]|\
 
 Inherits L<Tk::ListBrowser::BaseItem>.
 
-This module creates an object that holds all information of every entry.
-You will never need to create an item object yourself.
+This module creates an object that holds all information of
+a cell in a side column. You will never need to create an 
+item object yourself.
 
 Besides the options in it's parent, you can use the I<-data>, I<-image>,
 and I<-text> options.
@@ -44,80 +45,26 @@ and I<-text> options.
 
 sub new {
 	my $class = shift;
-	
 	my $self = $class->SUPER::new(@_);
-
-	$self->anchored(0);
-	$self->hidden(0) unless defined $self->hidden;
-	$self->opened(1);
 	$self->owner($self->listbrowser) unless defined $self->owner;
-	$self->{ANCHOR} = 0;
-	$self->{SELECTED} = 0;
-	
 	return $self
 }
 
-sub anchor {
-	my ($self, $flag) = @_;
-	my $c = $self->Subwidget('Canvas');
-	$flag = 1 unless defined $flag;
-	$self->{ANCHOR} = $flag;
-	if ($flag) {
-		$self->drawAnchor
-	} else {
-		$self->deleteAnchor
-	}
-	return $self->{ANCHOR}
-}
-
-sub anchored { return $_[0]->{ANCHOR} }
-
-sub canchor {
-	my $self = shift;
-	$self->{CANCHOR} = shift if @_;
-	return $self->{CANCHOR}
-}
-
-sub cguideH {
-	my $self = shift;
-	$self->{CGUIDEH} = shift if @_;
-	return $self->{CGUIDEH}
-}
-
-sub cguideV {
-	my $self = shift;
-	$self->{CGUIDEV} = shift if @_;
-	return $self->{CGUIDEV}
-}
-
 sub cimage {
 	my $self = shift;
 	$self->{CIMAGE} = shift if @_;
 	return $self->{CIMAGE}
 }
 
-sub cindicator {
-	my $self = shift;
-	$self->{CINDICATOR} = shift if @_;
-	return $self->{CINDICATOR}
-}
-
 sub clear {
 	my $self = shift;
 	my $c = $self->Subwidget('Canvas');
-	for ($self->canchor, $self->cimage, $self->ctext, $self->cguideH, $self->cguideV, $self->cindicator, $self->cselect) {
+	for ($self->cimage, $self->ctext) {
 		$c->delete($_) if defined $_;
 	}
-	$self->column(undef);
-	$self->row(undef);
 
-	$self->canchor(undef);
-	$self->cguideH(undef);
-	$self->cguideV(undef);
 	$self->cimage(undef);
 	$self->ctext(undef);
-	$self->cindicator(undef);
-	$self->cselect(undef);
 	$self->SUPER::clear;
 }
 
@@ -127,12 +74,6 @@ sub column {
 	return $self->{COLUMN}
 }
 
-sub cselect {
-	my $self = shift;
-	$self->{CSELECT} = shift if @_;
-	return $self->{CSELECT}
-}
-
 sub ctext {
 	my $self = shift;
 	$self->{CTEXT} = shift if @_;
@@ -145,16 +86,6 @@ sub data {
 	return $self->{DATA}
 }
 
-sub deleteAnchor {
-	my $self = shift;
-	my $r = $self->canchor;
-	return unless defined $r;
-	
-	my $c = $self->Subwidget('Canvas');
-	$c->delete($r);
-	$self->canchor(undef);
-}
-
 sub deleteImage {
 	my $self = shift;
 	my $i = $self->cimage;
@@ -175,29 +106,6 @@ sub deleteRect {
 	$self->crect(undef);
 }
 
-sub deleteSelect {
-	my $self = shift;
-	my $r = $self->cselect;
-	return unless defined $r;
-	
-	my $c = $self->Subwidget('Canvas');
-	$c->delete($r);
-	$self->cselect(undef);
-
-	my $t = $self->ctext;
-	$c->itemconfigure($t,
-		-fill => $self->cget('-foreground'),
-	) if defined $t;
-
-	my @columns = $self->columnList;
-	for (@columns) {
-		my $i = $self->itemGet($self->name, $_);
-		$c->itemconfigure($i->ctext, 
-			-fill => $self->cget('-foreground'),
-		) if (defined $i) and (defined $i->ctext);
-	}
-}
-
 sub deleteText {
 	my $self = shift;
 	my $t = $self->ctext;
@@ -322,27 +230,6 @@ sub draw {
 	$self->ismapped(1);
 }
 
-sub drawAnchor {
-	my $self = shift;
-	return unless $self->ismapped;
-	my @coords = $self->region;
-	$self->deleteAnchor;
-	my $c = $self->Subwidget('Canvas');
-	if ($self->listMode) {
-		my $lb = $self->listbrowser;
-		my ($cw) = $self->canvasSize;
-		my $yscroll = $self->Subwidget('YScrollbar');
-		$cw = $cw - $yscroll->width if $yscroll->ismapped;
-		$coords[0] = 0;
-		$coords[2] = $cw - 4;
-	}
-	my $a = $c->createRectangle(@coords,
-		-fill => undef,
-		-dash => [3, 2],
-	);
-	$self->canchor($a);
-}
-
 sub drawImage {
 	my $self = shift;
 	my $image = $self->image;
@@ -391,54 +278,6 @@ sub drawRect {
 	$c->raise($self->ctext) if defined $self->ctext;
 }
 
-sub drawSelect {
-	my $self = shift;
-	return unless $self->ismapped;
-	$self->deleteSelect;
-	my $c = $self->Subwidget('Canvas');
-	my $r = $self->crect;
-	my $t = $self->ctext;
-	my @columns = $self->columnList;
-	my @coords = $self->region;
-	if ($self->listMode) {
-		my $lb = $self->listbrowser;
-		my ($cw) = $self->canvasSize;
-		my $yscroll = $self->Subwidget('YScrollbar');
-		$cw = $cw - $yscroll->width if $yscroll->ismapped;
-		$coords[0] = 0;
-		$coords[2] = $cw - 4;
-	}
-
-	my $lb = $self->listbrowser;
-	my $si = Tk::ListBrowser::SelectXPM->new($lb);
-	my $a = $si->selectimage(@coords);
-	$self->cselect($a);
-	$c->raise($self->cimage, $a);
-	$c->raise($t, $a);
-	$c->raise($self->cindicator, $a);
-	$c->raise($self->cguideH, $a);
-	$c->raise($self->cguideV, $a);
-	$c->raise($self->canchor, $a) if defined $self->canchor;
-	my $next = $self->infoNext($self->name);
-	if (defined $next) {
-		my $n = $self->get($next);
-		$c->raise($n->cguideV, $a);
-	}
-	$c->itemconfigure($t,
-		-fill => $lb->cget('-selectforeground'),
-	);
-	for (@columns) {
-		my $i = $self->itemGet($self->name, $_);
-		if ((defined $i) and (defined $i->ctext)) {
-			$c->raise($i->ctext, $a) if defined $i->ctext;
-			$c->raise($i->cimage, $a) if defined $i->cimage;
-			$c->itemconfigure($i->ctext, 
-				-fill => $lb->cget('-selectforeground'),
-			);
-		}
-	}
-}
-
 sub drawText {
 	my $self = shift;
 	my $text = $self->textFormatted;
@@ -463,12 +302,6 @@ sub drawText {
 
 }
 
-sub hidden {
-	my $self = shift;
-	$self->{HIDDEN} = shift if @_;
-	return $self->{HIDDEN}
-}
-
 sub image {
 	my $self = shift;
 	if (@_) {
@@ -490,11 +323,6 @@ sub imageY {
 	return $self->{IMAGEY}
 }
 
-sub isentry {
-	my $self = shift;
-	return $self->owner eq $self->listbrowser;
-}
-
 sub inregion {
 	my ($self, $x, $y) = @_;
 	my ($cx, $cy, $cdx, $cdy) = $self->region;
@@ -505,6 +333,11 @@ sub inregion {
 	return 1
 }
 
+sub isentry {
+	my $self = shift;
+	return $self->owner eq $self->listbrowser;
+}
+
 sub minCellSize {
 	my ($self, $itemtype) = @_;
 	$itemtype = 'imagetext' unless defined $itemtype;
@@ -536,29 +369,6 @@ sub minCellSize {
 
 sub name { return $_[0]->{NAME} }
 
-sub opened {
-	my $self = shift;
-	$self->{OPENED} = shift if @_;
-	return $self->{OPENED}
-}
-
-sub openedparent {
-	my $self = shift;
-	my $name = $self->name;
-	my $p = $self->infoParent($name);
-	my $r = '';
-	if (defined $p) {
-		my $parent = $self->get($p);
-		if (($parent->openedparent) and ($parent->opened)) {
-			$r = 1
-		}
-	} else {
-		#is root always open
-		$r = 1
-	}
-	return $r
-}
-
 sub rectX {
 	my $self = shift;
 	$self->{RECTX} = shift if @_;
@@ -584,21 +394,6 @@ sub row {
 	return $self->{ROW}
 }
 
-sub select {
-	my ($self, $flag) = @_;
-	$flag = 1 unless defined $flag;
-	my $lb = $self->listbrowser;
-	if ($flag) {
-		return if $self->selected;
-		$self->drawSelect
-	} else {
-		$self->deleteSelect
-	}
-	$self->{SELECTED} = $flag;
-}
-
-sub selected { return $_[0]->{SELECTED} }
-
 sub tags {
 	my $self = shift;
 	my $owner = $self->owner;
